from rest_framework import generics, status, filters, views
from rest_framework.response import Response
from django.contrib.auth import get_user_model
from ..models import ReviewModel,OrganisationRatingModel
from django.db.models import Avg
from organisation.models import OrganisationModel
from slot_booking.models import DoctorBookingModel,OPDBookingModel,TestBookingModel
from django.db.models import Q
from .serializers import AppAPIsReviewModelListSerializer,OrganisationRatingModelSerializer



class AppAPIsReviewOrganisationModelListAPIView(generics.ListAPIView):
    serializer_class = AppAPIsReviewModelListSerializer

    def get_queryset(self):
        authenticated_user = self.request.user
        return ReviewModel.objects.filter(user=authenticated_user)

    def list(self, request, *args, **kwargs):
        serializer = self.serializer_class(self.get_queryset(), many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class AppAPIsRatingOrganisationModelListAPIView(generics.ListAPIView):
    serializer_class = AppAPIsReviewModelListSerializer

    def get_queryset(self):
        organisation_id = self.request.query_params.get('organisation_id')
        print(organisation_id)
        return ReviewModel.objects.filter(organisation=organisation_id)

    def list(self, request, *args, **kwargs):
        queryset = self.get_queryset()

        # Calculate the average rating using the aggregate function
        average_rating = queryset.aggregate(avg_rating=Avg('rating'))['avg_rating']

        # Serialize the queryset
        serializer = self.serializer_class(queryset, many=True)

        # Include the average_rating in the response data
        response_data = {
          
            'average_rating': average_rating,
        }

        return Response(response_data, status=status.HTTP_200_OK)


class ReviewRecordModelCreateAPIView(generics.CreateAPIView):

    queryset = ReviewModel.objects.all()
    serializer_class = AppAPIsReviewModelListSerializer

    def create(self, request, *args, **kwargs):
        authenticated_user = self.request.user
        auth_id = authenticated_user.id
        organisation_id = request.data.get('organisation')
        booking_id = request.data.get('booking_id')
        type = request.data.get('type')
        rating = request.data.get('rating')
        review = request.data.get('review')

        # Ensure 'organisation_id' is a valid integer
        try:
            organisation_id = int(organisation_id)
        except (ValueError, TypeError):
            return Response({'error': 'Invalid organisation ID'}, status=status.HTTP_400_BAD_REQUEST)

        # Fetch the OrganisationModel instance
        try:
            organisation_instance = OrganisationModel.objects.get(pk=organisation_id)
        except OrganisationModel.DoesNotExist:
            return Response({'error': 'Organisation not found'}, status=status.HTTP_404_NOT_FOUND)

        # Check if the user has already submitted a review for the organization
        existing_record = ReviewModel.objects.filter(user=auth_id, organisation=organisation_instance).first()

        # Use serializer for existing or new record
        if existing_record:
            # If the user has already submitted a review, update the existing record
            review_serializer = self.get_serializer(existing_record, data=request.data)
        else:
            # If there is no existing review, create a new one
            review_serializer = self.get_serializer(data=request.data)

        # Validate the review serializer
        review_serializer.is_valid(raise_exception=True)

        if existing_record:
            review_serializer.save()
        else:
            review_serializer.save(user=authenticated_user, organisation=organisation_instance)

        # Calculate the average rating for the organization
        average_rating = ReviewModel.objects.filter(organisation=organisation_instance).aggregate(avg_rating=Avg('rating'))['avg_rating']

        # Update or create OrganisationRatingModel record for the organization
        organisation_rating_instance, created = OrganisationRatingModel.objects.update_or_create(
            organisation=organisation_instance,
            defaults={'rating': average_rating}
        )

        

        if type == 'opd':
            opd_instance = OPDBookingModel.objects.filter(pk=booking_id).first()
            opd_instance.rating = rating
            opd_instance.review = review
            opd_instance.save()
        if type == 'test':
            test_instance = TestBookingModel.objects.filter(pk=booking_id).first()
            test_instance.rating = rating
            test_instance.review = review
            test_instance.save()
        if type == 'doctor':
            doctor_instance = DoctorBookingModel.objects.filter(pk=booking_id).first()
            doctor_instance.rating = rating
            doctor_instance.review = review
            doctor_instance.save()


        
        # Serialize the review and rating data
        response_data = {
            'success': True,
            'Review': "Review Saved Successfully"
        }

        return Response(response_data, status=status.HTTP_200_OK)
    

# class AppAPIsReviewOrganisationDoctorModelCreateAPI(generics.CreateAPIView):
#     queryset = ReviewOrganisationDoctorModel.objects.all()
#     serializer_class = AppAPIsReviewOrganisationDoctorModelCreateSerializer

#     def create(self, request):
#         try:
#             serializer = self.serializer_class(data=request.data, context={"request": request})
#             if serializer.is_valid():
#                 serializer.save()
#                 return Response({"message": "Created Successfully"}, status=status.HTTP_200_OK)
#             else:
#                 return Response({"message": serializer.errors}, status=status.HTTP_400_BAD_REQUEST)
#         except Exception as e:
#             return Response({"message": f"something went wrong, {e}"}, status=status.HTTP_400_BAD_REQUEST)


# class AppAPIsReviewOrganisationModelCreateAPIView(generics.CreateAPIView):
#     queryset = ReviewOrganisationModel.objects.all()
#     serializer_class = AppAPIsReviewOrganisationModelCreateSerializer

#     def create(self, request):
#         try:
#             serializer = self.serializer_class(data=request.data, context={"request": request})
#             if serializer.is_valid():
#                 serializer.save()
#                 return Response({"message": "Created Successfully"}, status=status.HTTP_200_OK)
#             else:
#                 return Response({"message": serializer.errors}, status=status.HTTP_400_BAD_REQUEST)
#         except Exception as e:
#             return Response({"message": f"something went wrong, {e}"}, status=status.HTTP_400_BAD_REQUEST)


# class AppAPIsReviewOrganisationDoctorModelListAPIView(generics.ListAPIView):
#     serializer_class = AppAPIsReviewModelDoctorListSerilizer

#     def get_queryset(self):
#         user = self.request.user
#         queryset = ReviewOrganisationDoctorModel.objects.filter(reviews__user=user)
#         reviews = ReviewModel.objects.filter(ReviewOrganisationDoctorModel_reviews__in=queryset).distinct()
#         return reviews.filter(user=user)

#     def list(self, request, *args, **kwargs):
#         serializer = self.serializer_class(self.get_queryset(), many=True)
#         return Response(serializer.data, status=status.HTTP_200_OK)


# class ReviewRecordModelListAPIView(generics.ListAPIView):
#     queryset = ReviewRecordModel.objects.all()
#     serializer_class = ReviewRecordModelSerializer

#     def list(self, request, *args, **kwargs):
#         instance, created = ReviewRecordModel.objects.get_or_create(review="__all__")
#         if instance.is_user_review:
#             instance.is_user_review = False
#         else:
#             instance.is_user_review = True
#         instance.save()
#         serializer = self.serializer_class(self.get_queryset(), many=True)
#         return Response({}, status=status.HTTP_200_OK)


       


# class AppAPIsReviewOrganisationDoctorModelCountListAPIView(generics.GenericAPIView):
#     serializer_class = AppAPIsReviewModelListSerializer

#     def get(self, request, org_id):
#         try:
#             organisation_instance = OrganisationModel.objects.get(pk=org_id)
#             # if organisation_instance.org_type=="DIAGNOSIS":
#             org_review_queryset = ReviewOrganisationDoctorModel.objects.filter(
#                 doctor__category__organisation=organisation_instance)
#             reviews = ReviewModel.objects.filter(
#                 ReviewOrganisationDoctorModel_reviews__in=org_review_queryset).distinct()
#             if reviews:
#                 rating_count = reviews.count()
#                 reviews_count = rating_count - (
#                             reviews.filter(description="").count() + reviews.filter(description=None).count())
#                 data = {
#                     "organisation_rating": organisation_instance.rating,
#                     "rating_count": rating_count,
#                     "reviews_count": reviews_count,
#                     "five_star": round((reviews.filter(rating=5).count()) / rating_count, 2),
#                     "four_star": round((reviews.filter(rating=4).count()) / rating_count, 2),
#                     "three_star": round((reviews.filter(rating=3).count()) / rating_count, 2),
#                     "two_star": round((reviews.filter(rating=2).count()) / rating_count, 2),
#                     "one_star": round((reviews.filter(rating=1).count()) / rating_count, 2),
#                 }
#                 return Response(data, status=status.HTTP_200_OK)
#             else:
#                 data = {
#                     "organisation_rating": organisation_instance.rating,
#                     "rating_count": 0,
#                     "reviews_count": 0,
#                     "five_star": 0,
#                     "four_star": 0,
#                     "three_star": 0,
#                     "two_star": 0,
#                     "one_star": 0,
#                 }
#                 return Response(data, status=status.HTTP_200_OK)
#             # elif organisation_instance.org_type=="HOSPITAL":
#             # pass
#             # else:
#             # return Response({"message": "Invalid organisation Id"},status=status.HTTP_400_BAD_REQUEST)
#         except Exception as e:
#             return Response({"message": f"Something went wrong : {e}"}, status=status.HTTP_400_BAD_REQUEST)


# class AppAPIsReviewOrganisationDoctorModelReviewListAPIView(generics.ListAPIView):
#     serializer_class = AppAPIsReviewModelListSerializer

#     def get_queryset(self):
#         org_id = self.kwargs['org_id']
#         organisation_instance = OrganisationModel.objects.get(pk=org_id)
#         org_review_queryset = ReviewOrganisationDoctorModel.objects.filter(
#             doctor__category__organisation=organisation_instance)
#         reviews = ReviewModel.objects.filter(ReviewOrganisationDoctorModel_reviews__in=org_review_queryset).distinct()
#         return reviews.exclude(Q(description=None) | Q(description='')).order_by('-rating')

#     def list(self, request, *args, **kwargs):
#         serializer = self.serializer_class(self.get_queryset(), many=True)
#         return Response(serializer.data, status=status.HTTP_200_OK)
